@@ -60,8 +60,9 @@ function log() {
     console.log("%s", tmpl.apply(null, arguments));
 }
 
-function make_code(ast, options) {
+function make_code(ast, options, expression) {
     var stream = U.OutputStream(options);
+    if (expression) ast = ast.clone(true).unwrap_expression();
     ast.print(stream);
     return stream.get();
 }
@@ -69,9 +70,7 @@ function make_code(ast, options) {
 function parse_test(file) {
     var script = fs.readFileSync(file, "utf8");
     try {
-        var ast = U.parse(script, {
-            filename: file
-        });
+        var ast = U.parse(script, { filename: file, module: "" });
     } catch (e) {
         console.error("Caught error while parsing tests in " + file);
         console.error(e);
@@ -98,14 +97,14 @@ function parse_test(file) {
             file: file,
             line: node.start.line,
             col: node.start.col,
-            code: make_code(node, { beautify: false })
+            code: make_code(node, { beautify: false }),
         }));
     }
 
     function read_string(stat) {
         if (stat.TYPE == "SimpleStatement") {
             var body = stat.body;
-            switch(body.TYPE) {
+            switch (body.TYPE) {
               case "String":
                 return body.value;
               case "Array":
@@ -142,7 +141,7 @@ function parse_test(file) {
                 ].indexOf(label.name) >= 0, tmpl("Unsupported label {name} [{line},{col}]", {
                     name: label.name,
                     line: label.start.line,
-                    col: label.start.col
+                    col: label.start.col,
                 }));
                 var stat = node.body;
                 if (label.name == "expect_exact" || label.name == "node_version") {
@@ -155,12 +154,12 @@ function parse_test(file) {
                         var ctor = global[body.expression.name];
                         assert.ok(ctor === Error || ctor.prototype instanceof Error, tmpl("Unsupported expect_stdout format [{line},{col}]", {
                             line: label.start.line,
-                            col: label.start.col
+                            col: label.start.col,
                         }));
                         test[label.name] = ctor.apply(null, body.args.map(function(node) {
                             assert.ok(node instanceof U.AST_Constant, tmpl("Unsupported expect_stdout format [{line},{col}]", {
                                 line: label.start.line,
-                                col: label.start.col
+                                col: label.start.col,
                             }));
                             return node.value;
                         }));
@@ -180,16 +179,23 @@ function parse_test(file) {
 
 // Try to reminify original input with standard options
 // to see if it matches expect_stdout.
-function reminify(orig_options, input_code, input_formatted, stdout) {
+function reminify(expression, orig_options, input_code, input_formatted, stdout) {
     for (var i = 0; i < minify_options.length; i++) {
         var options = JSON.parse(minify_options[i]);
-        if (options.compress) [
+        if (expression) {
+            if (!options.parse || typeof options.parse != "object") options.parse = {};
+            options.parse.expression = true;
+            if (options.compress == null) options.compress = {};
+            if (options.compress) {
+                if (typeof options.compress != "object") options.compress = {};
+                options.compress.expression = true;
+            }
+        }
+        [
             "keep_fargs",
             "keep_fnames",
         ].forEach(function(name) {
-            if (name in orig_options) {
-                options.compress[name] = orig_options[name];
-            }
+            if (name in orig_options) options[name] = orig_options[name];
         });
         var options_formatted = JSON.stringify(options, null, 4);
         options.validate = true;
@@ -214,7 +220,7 @@ function reminify(orig_options, input_code, input_formatted, stdout) {
         } else {
             var toplevel = sandbox.has_toplevel(options);
             var expected = stdout[toplevel ? 1 : 0];
-            var actual = sandbox.run_code(result.code, toplevel);
+            var actual = run_code(expression, result.code, toplevel);
             if (typeof expected != "string" && typeof actual != "string" && expected.name == actual.name) {
                 actual = expected;
             }
@@ -249,18 +255,23 @@ function reminify(orig_options, input_code, input_formatted, stdout) {
     return true;
 }
 
+function run_code(expression, code, toplevel) {
+    return sandbox.run_code(expression ? "console.log(" + code + ");" : code, toplevel);
+}
+
 function test_case(test) {
     log("    Running test [{name}]", { name: test.name });
     U.AST_Node.enable_validation();
     var output_options = test.beautify || {};
     var expect;
     if (test.expect) {
-        expect = make_code(to_toplevel(test.expect, test.mangle), output_options);
+        expect = to_toplevel(test.expect, test.mangle, test.expression);
+        expect = make_code(expect, output_options, test.expression);
     } else {
         expect = test.expect_exact;
     }
-    var input = to_toplevel(test.input, test.mangle);
-    var input_code = make_code(input);
+    var input = to_toplevel(test.input, test.mangle, test.expression);
+    var input_code = make_code(input, {}, test.expression);
     var input_formatted = make_code(test.input, {
         annotations: true,
         beautify: true,
@@ -270,7 +281,7 @@ function test_case(test) {
     });
     try {
         input.validate_ast();
-        U.parse(input_code);
+        U.parse(input_code, { expression: test.expression });
     } catch (ex) {
         log([
             "!!! Cannot parse input",
@@ -314,7 +325,7 @@ function test_case(test) {
         output.mangle_names(test.mangle);
         if (test.mangle.properties) U.mangle_properties(output, test.mangle.properties);
     }
-    var output_code = make_code(output, output_options);
+    var output_code = make_code(output, output_options, test.expression);
     U.AST_Node.log_function();
     if (expect != output_code) {
         log([
@@ -337,7 +348,7 @@ function test_case(test) {
     // expect == output
     try {
         output.validate_ast();
-        U.parse(output_code);
+        U.parse(output_code, { expression: test.expression });
     } catch (ex) {
         log([
             "!!! Test matched expected result but cannot parse output",
@@ -381,7 +392,7 @@ function test_case(test) {
         }
     }
     if (test.expect_stdout && (!test.node_version || semver.satisfies(process.version, test.node_version))) {
-        var stdout = [ sandbox.run_code(input_code), sandbox.run_code(input_code, true) ];
+        var stdout = [ run_code(test.expression, input_code), run_code(test.expression, input_code, true) ];
         var toplevel = sandbox.has_toplevel({
             compress: test.options,
             mangle: test.mangle
@@ -410,7 +421,7 @@ function test_case(test) {
             });
             return false;
         }
-        actual = sandbox.run_code(output_code, toplevel);
+        actual = run_code(test.expression, output_code, toplevel);
         if (!sandbox.same_stdout(test.expect_stdout, actual)) {
             log([
                 "!!! failed",
@@ -431,7 +442,7 @@ function test_case(test) {
             });
             return false;
         }
-        if (!reminify(test.options, input_code, input_formatted, stdout)) {
+        if (!reminify(test.expression, test.options, input_code, input_formatted, stdout)) {
             return false;
         }
     }
@@ -442,20 +453,29 @@ function tmpl() {
     return U.string_template.apply(null, arguments);
 }
 
-function to_toplevel(input, mangle_options) {
+function to_toplevel(input, mangle_options, expression) {
     if (!(input instanceof U.AST_BlockStatement)) throw new Error("Unsupported input syntax");
-    var directive = true;
     var offset = input.start.line;
     var tokens = [];
-    var toplevel = new U.AST_Toplevel(input.transform(new U.TreeTransformer(function(node) {
+    input.walk(new U.TreeWalker(function(node) {
         if (U.push_uniq(tokens, node.start)) node.start.line -= offset;
-        if (!directive || node === input) return;
-        if (node instanceof U.AST_SimpleStatement && node.body instanceof U.AST_String) {
-            return new U.AST_Directive(node.body);
-        } else {
+    }));
+    var toplevel;
+    if (!expression) {
+        var directive = true;
+        toplevel = new U.AST_Toplevel(input.transform(new U.TreeTransformer(function(node) {
+            if (!directive) return node;
+            if (node === input) return;
+            if (node instanceof U.AST_SimpleStatement && node.body instanceof U.AST_String) {
+                return new U.AST_Directive(node.body);
+            }
             directive = false;
-        }
-    })));
+        })));
+    } else if (input.body.length == 1) {
+        toplevel = input.body[0].wrap_expression();
+    } else {
+        throw new Error("Invalid expression");
+    }
     toplevel.figure_out_scope(mangle_options);
     return toplevel;
 }
